{ ******************************************************************
  This program computes the eigenvalues of a general square matrix
  (see eigensym.pas for a symmetric matrix).

  The file matrix1.inc is an example data file with N = 4
  ****************************************************************** }

program eigenval;

uses
  utypes, ueigval, ustrings, matutils;

{$define CompVector array[LbC..UbC : Integer] of Complex}

procedure WriteEigenValue(var Lambda : CompVector; I : Integer);
{ ----------------------------------------------------------------------
  Writes the I-th eigenvalue
  ---------------------------------------------------------------------- }
begin
  if Im(Lambda[I]) = 0.0 then
    WriteLn(FloatStr(Re(Lambda[I])))
  else
    WriteLn(CompStr(Lambda[I]));
end;

{$i matrix1.inc}

var
  Lambda  : array[1..N] of Complex;  { Eigenvalues }
  I       : Integer;                 { Loop variable }
  ErrCode : Integer;                 { Error code }

begin
  SetFormat(12, 6, False, True);
  WriteMatrix('Original matrix', A);

  { Compute eigenvalues }
  EigenVals(A, Lambda);

  ErrCode := MathErr;

  if ErrCode = 0 then
    begin
      WriteLn('Eigenvalues');
      WriteLn;
      for I := 1 to N do
        WriteEigenValue(Lambda, I);
    end
  else
    begin
      WriteLn('Unable to find eigenvalues Lambda[1] to Lambda[', -ErrCode, ']');
      WriteLn('Eigenvalues Lambda[', 1 - ErrCode, '] to Lambda[', N, ']:');
      for I := 1 - ErrCode to N do
        WriteEigenValue(Lambda, I);
    end;
end.
