{ ******************************************************************
  This program solves a system of linear equations (A * X = B) with
  a single constant vector by singular value decomposition.

  The file matrix3.inc is an example data file with N = 4
  ****************************************************************** }

program test_svd;

uses
  utypes, usvd, ustrings, matutils;

{$i matrix3.inc}

var
  U, V : array[1..N, 1..N] of Float;  { Matrices from SV decomp. }
  S    : array[1..N] of Float;        { Singular values }
  X    : array[1..N] of Float;        { Solution vector }
  I, J : Integer;                     { Loop variables }

begin
  { Initialize matrix U }
  for I := 1 to N do
    for J := 1 to N do
      U[I,J] := A[I,J];

  { Display data }
  SetFormat(12, 6, False, True);
  WriteMatrix('System matrix', A);
  WriteVector('Constant vector', B);

  { Perform SV decomposition of A. If successful, solve system }

  SV_Decomp(U, S, V);

  if MathErr = MatNonConv then
    begin
      WriteLn('Non-convergence!');
      Halt;
    end;

  SV_Solve(U, S, X, V, B);

  WriteVector('Solution vector', X);

  { Compute U*S*V' which should be equal to A }
  SV_Approx(U, S, V, A);
  WriteMatrix('Product U*S*V''', A);
end.
