{ ******************************************************************
  This program simulates a binormal distribution. The results are 
  stored in an output file (one random vector by line)
  ****************************************************************** }

program ranmul;

uses
  utypes, ucholesk, uranmult;

const
  N    = 2;         { Size of distribution }
  NSIM = 100;       { Number of simulations }
  R    = 0.285575;  { Correlation coefficient }

var  
  M : array[1..N] of Float = (309.332997, 1.292141);  { Means }
  S : array[1..N] of Float = ( 55.309195, 0.194529);  { S.D. }

  X : array[1..N] of Float;        { Random vector }
  V : array[1..N, 1..N] of Float;  { Variance-covariance matrix }
  L : array[1..N, 1..N] of Float;  { Cholesky factor of V }

  F    : Text;     { Output file }
  I, J : Integer;  { Loop variables }

begin
  for I := 1 to N do
    V[I,I] := Sqr(S[I]);      { Variances }
  
  V[1,2] := R * S[1] * S[2];  { Covariance }
  
  { Perform Cholesky decomposition of variance-covariance matrix }

  Cholesky(V, L);
  if MathErr = MatNotPD then
    begin
      WriteLn('Variance-covariance matrix is not positive definite.');
      Exit;
    end;

  Assign(F, 'ranmul.out');
  Rewrite(F);

  for I := 1 to NSIM do
    begin
      { Pick random vector }
      RanMult(L, M, X);

      { Output result to file }
      for J := 1 to N do
        Write(F, X[J]:12:6);
      Writeln(F);
    end;
  Close(F);
end.

