{ ******************************************************************
  Two-way analysis of variance (one observation per sample)
  ****************************************************************** }

program av2a;

uses
  utypes, uanova2, uinvbeta;

const
  NA    = 3;            { Number of modalities of factor A }
  NB    = 4;            { Number of modalities of factor B }
  Alpha = 0.05;         { Significance level }
  Prob  = 1.0 - Alpha;  { Probability }

{ The samples are stored in a matrix Z, such that
  Z[I, J] contains the observation for the I-th
  modality of factor A and the J-th modality of factor B }

var Z : array[1..NA, 1..NB] of Float =
((2, 1, 3, 1),
 (3, 2, 3, 2),
 (3, 4, 5, 3));

var
  S : array[1..NA, 1..NB] of Float;  { Standard deviations }

  V   : array[1..4] of Float;    { Variances (A, B, inter., resid.) }
  DoF : array[1..4] of Integer;  { Deg. freedom (A, B, inter., resid.) }
  F   : array[1..3] of Float;    { Variance ratios (A, B, inter.) }
  Fc  : array[1..3] of Float;    { Critical values }

  I, J : Integer;                { Loop variables }

begin
  { Compare means. The matrix of means is equal to the data matrix.
    The matrix of standard deviations will be ignored. }

  AnOVa2(1, Z, S, V, F, DoF);

  { Compute critical values }
  for I := 1 to 2 do
    Fc[I] := InvSnedecor(DoF[I], DoF[3], Prob);

  { Print results }
  WriteLn('Two-way ANOVA');
  WriteLn;
  WriteLn('Source         Variance    D.o.F.     F      F(p = ', Alpha:4:2, ')');
  WriteLn('--------------------------------------------------------');
  WriteLn('Factor A     ', V[1]:10:4, DoF[1]:10, F[1]:10:4, Fc[1]:10:4);
  WriteLn('Factor B     ', V[2]:10:4, DoF[2]:10, F[2]:10:4, Fc[2]:10:4);
  WriteLn('Interaction  ', V[3]:10:4, DoF[3]:10);
end.

