{ ******************************************************************
  Statistical distribution
  ****************************************************************** }

unit udistrib;

interface

uses
  utypes;

{$define Vector array[Lb..Ub : Integer] of Float}

{$define ClassVector array[LbC..UbC : Integer] of StatClass}

procedure Distrib(var X   : Vector;
                  A, B, H : Float;
                  var C   : ClassVector);
{ ------------------------------------------------------------------
  Distributes the values of array X[Lb..Ub] into M classes with
  equal width H, according to the following scheme:

              C[1]    C[2]                    C[M]
           ]-------]-------].......]-------]-------]
           A      A+H     A+2H                     B

  such that B = A + M * H
  ------------------------------------------------------------------ }

implementation

function NumCls(X, A, H : Float) : Integer;
{ Returns the index of the class containing X
  A is the lower bound of the first class
  H is the class width }
var
  Y : Float;
  I : Integer;
begin
  Y := (X - A) / H;
  I := Trunc(Y);
  if Y <> I then Inc(I);
  NumCls := I;
end;

procedure Distrib(var X   : Vector;
                  A, B, H : Float;
                  var C   : ClassVector);
var
  I, K, M, Nt : Integer;
begin
  M := Round((B - A) / H);

  if (LbC <> 1) or (UbC < M) then
    begin
      SetErrCode(MatErrDim);
      Exit;
    end;

  for K := 1 to M do
    C[K].N := 0;

  for I := Lb to Ub do
    begin
      K := NumCls(X[I], A, H);
      Inc(C[K].N);
    end;

  Nt := Ub - Lb + 1;

  for K := 1 to M do
    with C[K] do
      begin
        Inf := A + (K - 1) * H;
        Sup := Inf + H;
        F := N / Nt;
        D := F / H;
      end;
end;

end.
