{ ******************************************************************
  Eigenvalues and eigenvectors of a general square matrix
  ****************************************************************** }

unit ueigvec;

interface

uses
  utypes, ubalance, uelmhes, ueltran, uhqr2, ubalbak;

{$define Matrix array[Lb1..Ub1 : Integer; Lb2..Ub2 : Integer] of Float}

{$define CompVector array[LbC..UbC : Integer] of Complex}

{$define VMatrix array[LbV1..UbV1 : Integer; LbV2..UbV2 : Integer] of Float}

procedure EigenVect(var A      : Matrix;
                    var Lambda : CompVector;
                    var V      : VMatrix);

implementation

procedure EigenVect(var A      : Matrix;
                    var Lambda : CompVector;
                    var V      : VMatrix);
var
  I_low,
  I_igh : Integer;
  Scale : array[Lb1..Ub1] of Float;
  I_int : array[Lb1..Ub1] of Integer;

begin
  if (LbC <> Lb1) or (LbC <> Lb2) or (LbC <> LbV1) or (LbC <> LbV2) or
     (UbC <> Ub1) or (UbC <> Ub2) or (UbC <> UbV1) or (UbC <> UbV2) then
       begin
         SetErrCode(MatErrDim);
         Exit;
       end;

  Balance(A, I_low, I_igh, Scale);
  ElmHes(A, I_low, I_igh, I_int);
  Eltran(A, I_low, I_igh, I_int, V);
  Hqr2(A, I_low, I_igh, Lambda, V);

  if MathErr = 0 then
    BalBak(V, I_low, I_igh, Scale, Ub1);
end;

end.
