{ ******************************************************************
  Logarithms and exponentials (real variable)
  ****************************************************************** }

unit umath;

interface

uses
  utypes, usign;

function Expo(X : Float) : Float;     { Exponential }
function Exp2(X : Float) : Float;     { 2^X }
function Exp10(X : Float) : Float;    { 10^X }
function Log(X : Float) : Float;      { Natural log }
function Log2(X : Float) : Float;     { Log, base 2 }
function Log10(X : Float) : Float;    { Decimal log }
function LogA(X, A : Float) : Float;  { Log, base A }

implementation

  function Expo(X : Float) : Float;
  begin
    SetErrCode(FOk);
    if X < MinLog then
      Expo := DefaultVal(FUnderflow, 0.0)
    else if X > MaxLog then
      Expo := DefaultVal(FOverflow, MaxNum)
    else
      Expo := Exp(X);
  end;

  function Exp2(X : Float) : Float;
  var
    XLn2 : Float;
  begin
    SetErrCode(FOk);
    XLn2 := X * Ln2;
    if XLn2 < MinLog then
      Exp2 := DefaultVal(FUnderflow, 0.0)
    else if XLn2 > MaxLog then
      Exp2 := DefaultVal(FOverflow, MaxNum)
    else
      Exp2 := Exp(XLn2);
  end;

  function Exp10(X : Float) : Float;
  var
    XLn10 : Float;
  begin
    SetErrCode(FOk);
    XLn10 := X * Ln10;
    if XLn10 < MinLog then
      Exp10 := DefaultVal(FUnderflow, 0.0)
    else if XLn10 > MaxLog then
      Exp10 := DefaultVal(FOverflow, MaxNum)
    else
      Exp10 := Exp(XLn10);
  end;

  function Log(X : Float) : Float;
  begin
    SetErrCode(FOk);
    if X < 0.0 then
      Log := DefaultVal(FDomain, - MaxNum)
    else if X = 0.0 then
      Log := DefaultVal(FSing, - MaxNum)
    else
      Log := Ln(X);
  end;

  function Log10(X : Float) : Float;
  begin
    SetErrCode(FOk);
    if X < 0.0 then
      Log10 := DefaultVal(FDomain, - MaxNum)
    else if X = 0.0 then
      Log10 := DefaultVal(FSing, - MaxNum)
    else
      Log10 := Ln(X) * InvLn10;
  end;

  function Log2(X : Float) : Float;
  begin
    SetErrCode(FOk);
    if X < 0.0 then
      Log2 := DefaultVal(FDomain, - MaxNum)
    else if X = 0.0 then
      Log2 := DefaultVal(FSing, - MaxNum)
    else
      Log2 := Ln(X) * InvLn2;
  end;

  function LogA(X, A : Float) : Float;
  var
    Y : Float;
  begin
    Y := Log(X);
    if MathErr = FOk then
      if A = 1.0 then
        Y := DefaultVal(FSing, Sgn(Y) * MaxNum)
      else
        Y := Y / Log(A);
    LogA := Y;
  end;

end.
