{ ******************************************************************
  Mean and standard deviations
  ****************************************************************** }

unit umeansd;

interface

uses
  utypes;

{$define Vector array[Lb..Ub : Integer] of Float}

function MinVal(var X : Vector) : Float;
{ Minimum of sample X }

function MaxVal(var X : Vector) : Float;
{ Maximum of sample X }

function Mean(var X : Vector) : Float;
{ Mean of sample X }

function StDev(var X : Vector; M : Float) : Float;
{ Standard deviation estimated from sample X }

function StDevP(var X : Vector; M : Float) : Float;
{ Standard deviation of population }

implementation

function MinVal(var X : Vector) : Float;
var
  Xmin : Float;
  I    : Integer;
begin
  Xmin := X[Lb];

  for I := Succ(Lb) to Ub do
    if X[I] < Xmin then Xmin := X[I];

  MinVal := Xmin;
end;

function MaxVal(var X : Vector) : Float;
var
  Xmax : Float;
  I    : Integer;
begin
  Xmax := X[Lb];

  for I := Succ(Lb) to Ub do
    if X[I] > Xmax then Xmax := X[I];

  MaxVal := Xmax;
end;

function Mean(var X : Vector) : Float;
var
  SX : Float;
  I  : Integer;
begin
  SX := 0.0;

  for I := Lb to Ub do
    SX := SX + X[I];

  Mean := SX / (Ub - Lb + 1);
end;

function StDev(var X : Vector; M : Float) : Float;
var
  D, SD, SD2, V : Float;
  I, N          : Integer;
begin
  N := Ub - Lb + 1;

  SD  := 0.0;  { Sum of deviations (used to reduce roundoff error) }
  SD2 := 0.0;  { Sum of squared deviations }

  for I := Lb to Ub do
  begin
    D := X[I] - M;
    SD := SD + D;
    SD2 := SD2 + Sqr(D)
  end;

  V := (SD2 - Sqr(SD) / N) / (N - 1);  { Variance }
  StDev := Sqrt(V);
end;

function StDevP(var X : Vector; M : Float) : Float;
var
  D, SD, SD2, V : Float;
  I, N          : Integer;
begin
  N := Ub - Lb + 1;

  SD  := 0.0;  { Sum of deviations (used to reduce roundoff error) }
  SD2 := 0.0;  { Sum of squared deviations }

  for I := Lb to Ub do
  begin
    D := X[I] - M;
    SD := SD + D;
    SD2 := SD2 + Sqr(D)
  end;

  V := (SD2 - Sqr(SD) / N) / N;  { Variance }
  StDevP := Sqrt(V);
end;

end.
