{ ******************************************************************
  Multinormal distribution
  ****************************************************************** }

unit uranmult;

interface

uses
  utypes, urangaus;

{$define Vector array[Lb..Ub : Integer] of Float}

{$define XVector array[LbX..UbX : Integer] of Float}

{$define Matrix array[Lb1..Ub1 : Integer; Lb2..Ub2 : Integer] of Float}

procedure RanMult(var M : Vector; var L : Matrix; var X : XVector);
{ ------------------------------------------------------------------
  Generates a random vector X from a multinormal distribution.
  M is the mean vector, L is the Cholesky factor (lower triangular)
  of the variance-covariance matrix.
  ------------------------------------------------------------------ }

procedure RanMultIndep(var M, S, X : Vector);
{ ------------------------------------------------------------------
  Generates a random vector X from a multinormal distribution with
  uncorrelated variables. M is the mean vector, S is the vector
  of standard deviations.
  ------------------------------------------------------------------ }

implementation

procedure RanMult(var M : Vector; var L : Matrix; var X : XVector);
var
  I, J : Integer;
  U    : array[Lb..Ub] of Float;
begin
  { Form a vector U of independent standard normal variates }
  for I := Lb to Ub do
    U[I] := RanGaussStd;

  { Form X = M + L * U, which follows the multinormal distribution }
  for I := Lb to Ub do
    begin
      X[I] := M[I];
      for J := Lb to I do
        X[I] := X[I] + L[I,J] * U[J]
    end;

end;

procedure RanMultIndep(var M, S, X : Vector);
var
  I : Integer;
begin
  for I := Lb to Ub do
    X[I] := RanGauss(M[I], S[I])
end;

end.
