{ ******************************************************************
  Roots of a polynomial
  ****************************************************************** }

unit urootpol;

interface

uses
  utypes, urtpol1, urtpol2, urtpol3, urtpol4, ubalance, uhqr;

{$define Vector array[Lb..Ub : Integer] of Float}

{$define CompVector array[LbC..UbC : Integer] of Complex}

function RootPol(var Coef : Vector; var Z : CompVector) : Integer;
{ ------------------------------------------------------------------
  Solves the polynomial equation:
  Coef(0) + Coef(1) * Z + Coef(2) * Z^2 + ...
                        + Coef(Deg) * Z^Deg = 0

  Coef[0..Deg] = polynomial coefficients (real)
  Z[1..Deg]    = roots (complex)
  ------------------------------------------------------------------ }

implementation

function RootPolMat(var Coef : Vector; var Z : CompVector) : Integer;
{ Roots of polynomial from companion matrix }

var
  A      : array[1..Ub, 1..Ub] of Float;  { Companion matrix }
  Scale  : array[1..Ub] of Float;         { Used by Balance }
  Lo, Hi : Integer;                       { Used by Balance }
  I, J   : Integer;                       { Loop variables  }
  Nr     : Integer;                       { Number of real roots }

begin
  { Set up the companion matrix }
  for J := 1 to Ub do
    A[1,J] := - Coef[Ub - J] / Coef[Ub];

  for I := 2 to Ub do
    for J := 1 to Ub do
      if I - 1 = J then A[I,J] := 1.0 else A[I,J] := 0.0;

  { The roots of the polynomial are the
    eigenvalues of the companion matrix }
  Balance(A, Lo, Hi, Scale);
  Hqr(A, Lo, Hi, Z);

  if MathErr <> 0 then
    begin
      RootPolMat := MathErr;
      Exit;
    end;

  { Count real roots }
  Nr := 0;
  for I := 1 to Ub do
    if Im(Z[I]) = 0.0 then Inc(Nr);

  RootPolMat := Nr
end;

function RootPol(var Coef : Vector; var Z : CompVector) : Integer;
begin
  if (Lb <> 0) or (LbC <> 1) or (Ub <> UbC) then
    begin
      RootPol := MatErrDim;
      Exit;
    end;

  case Ub of
    1 : RootPol := RootPol1(Coef, Z);
    2 : RootPol := RootPol2(Coef, Z);
    3 : RootPol := RootPol3(Coef, Z);
    4 : RootPol := RootPol4(Coef, Z);
  else
        RootPol := RootPolMat(Coef, Z);
  end;
end;

end.
