{ ******************************************************************
  Student t-test for paired samples
  ****************************************************************** }

unit ustdpair;

interface

uses
  utypes, usign, umeansd;

{$define Vector array[Lb..Ub : Integer] of Float}

procedure StudPaired(var X, Y : Vector;
                     var T    : Float;
                     var DoF  : Integer);
{ ------------------------------------------------------------------
  Student t-test for paired samples
  ------------------------------------------------------------------
  Input parameters : X, Y = samples
  Output parameters: T    = Student's t
                     DoF  = degrees of freedom
  ------------------------------------------------------------------ }

implementation

procedure StudPaired(var X, Y : Vector;
                     var T    : Float;
                     var DoF  : Integer);

var
  D      : array[Lb..Ub] of Float;  { Differences between samples  }
  MD, SD : Float;                   { Mean & std.dev. of differences }
  N      : Integer;                 { Sample size }
  I      : Integer;                 { Loop variable }

begin
  for I := Lb to Ub do
    D[I] := X[I] - Y[I];

  MD := Mean(D);
  SD := StDev(D, MD);

  if SD = 0.0 then
    begin
      T := Sgn(MD) * MaxNum;
      SetErrCode(FSing);
    end;

  DoF := Ub - Lb;  { N - 1 }
  N := DoF + 1;
  T := MD * Sqrt(N) / SD;
end;

end.
